<?php


namespace JtF\FormReformDisplay;

defined('C5_EXECUTE') or die('Access Denied.');

use Concrete\Core\File\Image\Thumbnail\Type\Type as ThumbnailType;
use Illuminate\Support\Arr;
use Concrete\Core\Utility\Service\Arrays as ArraysService;
use Concrete\Core\Page\Page;
use Concrete\Core\File\File;
use Concrete\Core\User\User;
use DateTime;

use JtF\FormReform\DataValues;
use JtF\FormReform\DataValueExtractor\DataValuesForm;
use JtF\FormReformDisplay\DateFormatter;
use JtFResources\SystemBase;


/*
 * Software by John Liddiard (aka JohntheFish)
 * www.c5magic.co.uk
 *
 * This package contains software copyright and proprietary to John Liddiard
 *
 */

class DisplayItemFormatter
{
    protected $data_key;
    protected $result; // entire result including metadata
    protected $form_data; // form data, usually a record within the $result
    protected $th;
    protected $ah;
    protected $data_values_form;

    public function __construct($data_key, $result, $form_data = null, $date_format = 'Y-m-d H:i:s')
    {
        $this->data_key = $data_key;
        $this->result = $result;
        $this->form_data = $form_data;
        $this->date_format = $date_format;

        $this->th = app()->make('helper/text');
        $this->ah = app()->make(ArraysService::class);
        $data_vaues = new DataValues(null,);
        $this->data_values_form = new DataValuesForm($data_vaues);
    }

    /*
     * General data formatting
     */
    public function formatDataItem($format_hint = false, $format_helpers = null)
    {
        if ($format_hint === 1 || $format_hint === '1' || $format_hint === true) {
            $format_hint = 'html';
        } elseif (!is_scalar($format_hint)) {
            $format_hint = '';
        }

        if (is_array($this->form_data) && isset($this->form_data[$this->data_key])) {
            $data_item = $this->form_data[$this->data_key];

        } elseif (is_array($this->result) && isset($this->result[$this->data_key])) {
            $data_item = $this->result[$this->data_key];

        } elseif (is_object($this->result) && isset($this->result->{$this->data_key})) {
            $data_item = $this->result->{$this->data_key};
        }

        /*
         * Dig into an array by index or dot notation
         */

        if (is_array($this->form_data) && !isset($data_item) && (strpos($this->data_key, '[') !== false || strpos($this->data_key, '.') !== false)) {
            $keylist = str_replace(']', '', $this->data_key);
            $keylist = preg_split("/[\[\]\.]+/", trim($keylist, '['));
            $v2 = $this->ah->get($this->form_data, $keylist);
            if (!is_null($v2)) {
                $data_item = $v2;
            }
        }

        if (is_array($this->form_data) && !isset($data_item) && strpos($this->data_key, '.') !== false) {
            $dot_data = Arr::dot($this->form_data);
            $v2 = [];
            foreach ($dot_data as $k => $v) {
                if ($this->data_key === $k) {
                    $v2[] = $v;
                }
                if (strpos($this->data_key, $k) === 0) {
                    $k2 = substr($k, strlen($this->data_key));
                    if (substr($k2, 0, 1) === '.') {
                        $k2 = substr($k2, 1);
                        $v2[$k2] = $v;
                    }
                }
            }
            if (!empty($v2)) {
                $data_item = $v2;
            }
        }

        if (!isset($data_item)) {
            $data_item = "";
        }

        /*
         * Whether we have an array, json, or a scalar, at this stage we want
         * to get it into a flattened array we can loop through.
         */
        if (!is_array($data_item)) {
            $data_item_b = SystemBase::unSquish_s($data_item);
            if (!empty($data_item_b) && is_array($data_item_b)) {
                $data_items = Arr::dot($data_item_b);
            } else {
                $data_items = Arr::dot([$data_item]);
            }
        } else {
            $data_items = Arr::dot($data_item);
        }

        $item_class = 'form-reform-display-format-item';
        $op = [];
        foreach ($data_items as $ix => $current_item) {
            if (!is_null($current_item) &&
                !empty($format_hint) &&
                is_array($format_helpers) &&
                array_key_exists($format_hint, $format_helpers)
            ) {
                /*
                 * This allows other blocks to extend the formatting options by passing a
                 * list of format helpers as
                 * [
                 *     'format_name'=>[object, method] // or other callable
                 * ]
                 */
                $format_helper = $format_helpers[$format_hint];
                if (is_callable($format_helper)) {
                    $op[] = call_user_func($format_helper, $current_item);
                    continue;
                }
            }

            if (!empty($current_item) &&
                !empty($format_hint) &&
                (
                    DateFormatter::isDateTime($current_item) ||
                    $format_hint === 'date_time'
                )
            ) {
                $df = app()->make(DateFormatter::class);
                $op[] = $df->format($current_item, $this->date_format);
                continue;
            }

            if (!empty($current_item) &&
                !empty($format_hint) &&
                (
                    preg_match("/^h[123456]{1}$/", $format_hint) ||
                    $format_hint === 'div' ||
                    $format_hint === 'span'
                )
            ) {
                $op[] = sprintf('<%s class="%s">%s</%s>', $format_hint, $item_class, h($current_item), $format_hint);
                continue;
            }

            if (!empty($current_item) &&
                !empty($format_hint) &&
                $format_hint === 'cid' &&
                is_numeric($current_item)
            ) {
                $page = Page::getByID($current_item);
                if (is_object($page)) {
                    $op[] = sprintf('<a class="%s" href="%s">',
                                    $item_class,
                                    $page->getCollectionLink()
                        ) . $page->getCollectionName() . '</a>';
                    continue;
                }
            }

            if (!empty($current_item) &&
                !empty($format_hint) &&
                $format_hint === 'fid' &&
                is_numeric($current_item)
            ) {
                $f = File::getByID($current_item);
                if (is_object($f)) {
                    $fv = $f->getVersion();
                    if (is_object($fv)) {
                        $op[] = sprintf('<a class="%s" href="%s">',
                                        $item_class,
                                        $fv->getDownloadURL(),
                            ) . $fv->getTitle() . '</a>';
                        continue;
                    }
                }
            }

            if (!empty($current_item) &&
                !empty($format_hint) &&
                $format_hint === 'img' &&
                is_numeric($current_item . '')
            ) {
                $f = File::getByID($current_item);
                if (is_object($f)) {
                    $fv = $f->getVersion();
                    $fv = $f->getVersion();
                    if (is_object($fv)) {
                        $op[] = $fv->getListingThumbnailImage();
                        continue;
                    }
                }
            }

            if (!empty($current_item) &&
                !empty($format_hint) &&
                ($format_hint === 'html' || $format_hint === 'img') &&
                is_string($current_item) &&
                preg_match("{^data:(image/.+?);base64,(.*)$}", $current_item, $m)
            ) {
                $op[] = sprintf('<img class="%s" src="%s" style="object-fit:contain;max-width:100%%;max-height:auto;">',
                                $item_class, $current_item
                );
                continue;
            }
            $op[] = $this->data_values_form->format($current_item);
            continue;
        }
        if (!empty($op)) {
            if (empty($format_hint)) {
                return $this->data_values_form->format($op);
            } else {
                return implode("\n", $op);
            }
        }

        return $this->data_values_form->format($data_item);

    }

    /*
     * Some special formatting rules for metadata.
     */
    public function formatMetadataItem()
    {
        $k_getter = 'get' . $this->th->camelCase($this->data_key);
        $item_class = 'form-reform-display-format-item';

        if (is_object($this->result) && is_callable([$this->result, $k_getter])) {
            $data_item = $this->result->$k_getter();

        } elseif (is_object($this->result) && isset($this->result->$this->data_key)) {
            $data_item = $this->result->$this->data_key;

        } elseif (is_array($this->result) && array_key_exists($this->data_key, $this->result)) {
            $data_item = $this->result[$this->data_key];

        } elseif (is_array($this->result) && array_key_exists($this->th->camelCase($this->data_key), $this->result)) {
            $data_item = $this->result[$this->th->camelCase($this->data_key)];

        } else {
            $data_item = "";
        }

        if ($data_item instanceof DateTime || DateFormatter::isDateTime($data_item)) {
            $df = app()->make(DateFormatter::class);
            $data_item = $df->format($data_item, $this->date_format);
        } elseif ($this->data_key === 'cid') {
            $page = Page::getByID($data_item);
            if (is_object($page)) {
                $data_item = sprintf('<a class="%s" href="%s">',
                                     $item_class,
                                     $page->getCollectionLink()
                    ) . $page->getCollectionName() . '</a>';
            }
        } elseif ($this->data_key === 'uid' || $this->data_key === 'uid_modified') {
            $u = User::getByUserID($data_item);
            if (is_object($u)) {
                $data_item = $u->getUserName();
            } else {
                $data_item = t('Guest');
            }
        }
        if (!is_scalar($data_item)) {
            $data_item = h(json_encode($data_item, JSON_UNESCAPED_SLASHES + JSON_UNESCAPED_UNICODE));
        }
        return $data_item;
    }


    public static function dataFormatOptions()
    {
        return [
            ''          => t('none'),
            'h1'        => t('H1'),
            'h2'        => t('H2'),
            'h3'        => t('H3'),
            'h4'        => t('H4'),
            'h5'        => t('H5'),
            'h6'        => t('H6'),
            'span'      => t('span'),
            'div'       => t('div'),
            'html'      => t('Allow html'),
            'cid'       => t('page id link'),
            'fid'       => t('file id link'),
            'img'       => t('file id thumbnail'),
            'date_time' => t('date-time'),
        ];
    }
}