<?php


namespace JtF\FormReformDisplay;

defined('C5_EXECUTE') or die('Access Denied.');

use JtFResources\SystemBase;

/*
 * Software by John Liddiard (aka JohntheFish)
 * www.c5magic.co.uk
 *
 * This package contains software copyright and proprietary to John Liddiard
 *
 */

class ResultIDEncryption
{

    /*
     * Needs to be repeatable within a request, so we have a static cache
     * Note extra data to provide additional verification that an old encrypted
     * ID is not being hacked.
     */

    static $enc_list = [];

    public static function id_encode($id)
    {
        $r_list = [];
        if (is_array($id)) {
            foreach ($id as $id_n) {
                $r_list[] = self::id_encode($id_n);
            }
            return $r_list;
        }

        if (empty(self::$enc_list[$id])) {
            $token = app()->make('helper/validation/token');
            $token_code = $token->generate(__CLASS__);
            self::$enc_list[$id] = base64_encode(
                SystemBase::encrypt_s(
                    [
                        'id'  => $id,
                        'vt'  => $token_code
                    ]
                )
            );
        }
        return self::$enc_list[$id];
    }

    public static function id_decode($encoded_id)
    {
        if (is_array($encoded_id)) {
            $r_list = [];
            foreach ($encoded_id as $enc_id) {
                $r_list[] = self::id_decode($enc_id);
            }
            return $r_list;
        }
        $r = SystemBase::decrypt_s(base64_decode($encoded_id));
        $token = app()->make('helper/validation/token');
        if (is_array($r) && !empty($r['id']) &&
            !empty($r['vt']) && $token->validate(__CLASS__, $r['vt'])
        ) {
            return $r['id'];
        }
    }

    public static function id_recode($encoded_id)
    {
        $id = self::id_decode($encoded_id);
        if (!empty($id)) {
            return self::id_encode($id);
        } else {
            return $encoded_id;
        }
    }
}